<?php

/**
 * Modelová třída doplňku prezentujícího vývoj ceny hliníku
 */
class AluPrice extends Object
{

	const CACHE_ALUPRICE = 'aluPrice';
	const CACHE_ALUPRICE_LAST = 'aluPrice.last';
	const CACHE_ALUPRICE_DATE = 'aluPrice.date';

	const CACHE_REFRESH = 21600;

	const ERRMODE_SILENT = 1;
	const ERRMODE_EXCEPTIONS = 2;

	private $cacheProperty;

	private $priceSource;

	private $errorMode = 1;


	public function __construct($errorMode = null)
	{
		if ($errorMode !== null) {
			$this->errorMode = $errorMode;
		}
	}


	public function setPriceSource(IAluPriceSource $priceSource)
	{
		$this->priceSource = $priceSource;
	}


	public function setErrorMode($errorMode)
	{
		$this->errorMode = $errorMode;
	}


	public function getCache()
	{
		if ($this->cacheProperty === null) {
			$this->cacheProperty = Environment::getCache('BP');
		}
		return $this->cacheProperty;
	}


	public function getAluPrice($throwExceptions = false)
	{
		if (isset($this->cache[self::CACHE_ALUPRICE])) {
			return $this->cache['aluPrice'];
		} else {
			if ($this->priceSource === null) {
				if ($this->error(new ApplicationException('Nebyl nastaven žádný zdroj ceny hliníku.'))) return;
			}
			try {
				$price = $this->priceSource->getPrice();
				$this->cache->save(self::CACHE_ALUPRICE, $price, array(
					'expire' => time() + self::CACHE_REFRESH,
				));
				$this->cache->save(self::CACHE_ALUPRICE_DATE, date('j.n.Y'));
				$this->cache->save(self::CACHE_ALUPRICE_LAST, $price);
				
			} catch (IOException $e) {
				if (!isset($this->cache[self::CACHE_ALUPRICE_LAST])) {
					$this->cache[self::CACHE_ALUPRICE_LAST] = null;
				}
				$this->cache->save(self::CACHE_ALUPRICE, $this->cache[self::CACHE_ALUPRICE_LAST], array(
					'expire' => time() + self::CACHE_REFRESH,
				));
				return $this->cache[self::CACHE_ALUPRICE_LAST];
			}
		}
		return $this->cache[self::CACHE_ALUPRICE_LAST];
	}


	public function getAluPriceDate()
	{
		if (isset($this->cache[self::CACHE_ALUPRICE_DATE])) {
			return $this->cache[self::CACHE_ALUPRICE_DATE];
		} else {
			return false;
		}
	}


	public function getMonthAverageAluPrice($year, $month)
	{
		try {
			return dibi::fetchSingle('SELECT [price] FROM [aluprice] WHERE [year] = %i', $year, 'AND [month] = %i', $month);
		} catch (Exception $e) {
			if ($this->error($e)) return;
		}
	}


	public function getLastActualMonthAverageAluPrice()
	{
		try {
			return dibi::fetchSingle('
				SELECT [price]
				FROM [aluprice]
				WHERE [year] <= DATE_FORMAT(NOW(), "%Y") AND [month] <= DATE_FORMAT(NOW(), "%m")
				ORDER BY [year] DESC, [month] DESC
				LIMIT 1
			');
		} catch (Exception $e) {
			if ($this->error($e)) return;
		}
	}


	public function generateGraph()
	{
		$description = array();
		$description['Axis']['X'] = 'Datum';
		$description['Axis']['Y'] = 'Cena v USD';
		$description['Values'][] = 'Serie1';
		$description['Format']['X'] = 'date';
		$description['Format']['Y'] = 'number';
		$description['Position'] = 'Name';
		$description['Unit']['X'] = '';
		$description['Unit']['Y'] = '';

		$data = array();
		try {
			$rawData = array_reverse(dibi::fetchAll('SELECT * FROM [aluprice] WHERE [year] <= DATE_FORMAT(NOW(), "%Y") AND [month] <= DATE_FORMAT(NOW(), "%m") ORDER BY [year] DESC, [month] DESC LIMIT 7'));
		} catch (Exception $e) {
			if ($this->error($e)) return;
		}
		if (empty($rawData)) return;

		foreach ($rawData as $item) {
			$data[] = array('Name' => mktime(0, 0, 0, $item['month'], 1, $item['year']), 'Serie1' => $item['price']);
		}		

		$chart = new pChart(204,220);

		$chart->setColorPalette(0,255,0,0);

		$chart->setFontProperties(LIBS_DIR.'/pChart/Fonts/tahoma.ttf',7);
		$chart->drawBackground(241,241,241);
		$chart->setGraphArea(47,20,195,160);
		$chart->drawGraphArea(254,254,254);

		$chart->setDateFormat('n/Y');
		$chart->drawScale($data,$description,SCALE_NORMAL,120,120,120,TRUE,90,0);
		$chart->drawGrid(2,false,179,179,179,255);

		$chart->drawLineGraph($data,$description);
		$chart->drawPlotGraph($data,$description,3,2,255,255,255);

		$chart->Render(WWW_DIR.'/graph/aluprices_cs.png');
	}


	public function findYearPrices($year)
	{
		try {
			return dibi::fetchAll('SELECT [month], [price] FROM [aluprice] WHERE [year] = %i', $year, 'ORDER BY [month]');
		} catch (DibiException $e) {
			if ($this->error($e)) return;
		}
	}


	public function insertPrice($params)
	{
		try {
			dibi::query('INSERT INTO [aluprice]', $params, 'ON DUPLICATE KEY UPDATE [price] = %i', $params['price']);
		} catch (DibiException $e) {
			if ($this->error($e)) return;
		}
	}


	public function deletePrice($year, $month)
	{
		try {
			dibi::query('DELETE FROM [aluprice] WHERE [year] = %i', $year, 'AND [month] = %i', $month);
		} catch (DibiException $e) {
			if ($this->error($e)) return;
		}
	}


	protected function error($e)
	{
		if ($this->errorMode === self::ERRMODE_EXCEPTIONS) {
			throw $e;
		} elseif ($this->errorMode === self::ERRMODE_SILENT) {
			return true;
		}
	}

}