<?php

/**
 *
 *
 */

class SiteTree extends SiteStateClientBaseModel
{

	private $cacheProperty;


	public function getCache()
	{
		if ($this->cacheProperty === null) {
			$this->cacheProperty = Environment::getCache('CMF.Core');
		}
		return $this->cacheProperty;
	}



	public function getAncestors($includeActivePage = false, $pageId = null)
	{
		if ($pageId === null) {
			$pageId = $this->siteState['pageId'];
		}

		$cacheKey = 'Ancestors.'.strval($pageId);

		$cache = Environment::getCache('CMF.Core');

		if (isset($this->cache[$cacheKey])) {
			$ancestors = $this->cache[$cacheKey];
		} else {

			$page = dibi::fetch('
				SELECT [id] [pageId], [name], [homepage], [lang], [url], [family], [left], [right]
				FROM [page]
				WHERE [id] = %i', $pageId
			);

			if ($page === false) {
				throw new InvalidArgumentException('Stránka, ke které se hledali předci, nebyla nalezena.');
			}
			$ancestors = dibi::query('
				SELECT [id] [pageId], [name], [homepage], [lang], [url]
				FROM [page]
				WHERE [left] < %i', $page['left'], ' AND [right] > %i', $page['right'], ' AND [family] = %i', $page['family'], '
				ORDER BY [left]
			')->fetchAssoc('pageId');

			foreach (array('family', 'left', 'right') as $key) {
				unset($page[$key]);
			}
			$ancestors[] = $page;

			$this->cache->save($cacheKey, $ancestors, array(
				Cache::TAGS => array('SiteTree.'.$page['lang'])
			));
		}

		if ($includeActivePage === false) {
			array_pop($ancestors);
		}
		return $ancestors;
	}



	public function getSiblings($parentId = null, $lang = null, $currentId = null)
	{
		$query = dibi::select('[ord], [name]')->from('page');
		if ($parentId === null) {
			if ($lang === null) {
				throw new InvalidArgumentException('Pro získání stránek na první úrovni je zapotřebí znát jazyk.');
			}
			$query->where('[parent] IS NULL AND [lang] = %s', $lang);
		} else {
			$query->where('[parent] = %i', $parentId);
		}
		if ($currentId !== null) {
			$query->and('[id] != %i', $currentId);
		}
		return $query->orderBy('ord')->fetchPairs();
	}


	public function getForegoingSibling($pageId)
	{

		$page = dibi::query('SELECT [parent], [ord] FROM [page] WHERE [id] = %i', $pageId)->fetch();
		if ($page === false) {
			throw new InvalidArgumentException('Požadovaná stránka nebyla nalezena.');
		}

		$query = dibi::select('ord')->from('page');
		if ($page['parent'] === null) {
			$query->where('[parent] IS NULL');
		} else {
			$query->where('[parent] = %i', $page['parent']);
		}
		$sibling = $query->and('[ord] < %i', $page['ord'])->orderBy('ord', dibi::DESC)->limit(1)->execute()->fetchSingle();

		if ($sibling === false) {
			return null;
		}
		return $sibling;

	}



	public function getSiteTree($markTrail = false, $activePageId = null, $lang = null)
	{

		if ($lang === null) {
			$lang = $this->siteState['lang'];
		}

		$siteTree = array();

		$cache = Environment::getCache('CMF.Core');

		if (isset($this->cache['SiteTree.'.$lang])) {
			$siteTree = $this->cache['SiteTree.'.$lang];
		} else {

			foreach (dibi::query('
				SELECT [id] [pageId], [family], [left], [right]
				FROM [page]
				WHERE [parent] IS NULL AND [lang] = %s', $lang, '
				ORDER BY [ord]
			') as $patriarchs) {

				$pages = dibi::fetchAll('
					SELECT [id] [pageId], [page.name], [page-type.name] [typeName], [homepage], /*[lang],*/ [url], [parent], [page.ord], [left], [right], [published], [shownInMenu], [presenter], [currentStep]
					FROM [page] JOIN [page-type] ON [page.page-type_type] = [page-type.type]
					WHERE [left] BETWEEN %i', $patriarchs['left'], ' AND %i', $patriarchs['right'], '
					AND [family] = %i', $patriarchs['family'], '
					ORDER BY [left]'
				);

				$siteTree[] = $this->treeToArray($pages);
			}

			$this->cache->save('SiteTree.'.$lang, $siteTree, array(
				Cache::TAGS => array('SiteTree.'.$lang)
			));

		}

		if ($markTrail !== false) {
			if ($activePageId === null) {
				$activePageId = $this->siteState['pageId'];
			}
			$ancestors = array_keys($this->getAncestors(false, $activePageId));
			$this->markTrail($siteTree, $ancestors, $activePageId);
		}
		return $siteTree;

	}



	public function getSiteSubtree($parentId = null, $depth = null, $markTrail = false, $activePageId = null, $lang = null)
	{

		if ($parentId === null and $lang === null) {
			$lang = $this->siteState['lang'];
		}

		// pokud není definován rodič a není omezena hloubka, je požadován celý strom webu
		if ($parentId === null and $depth === null) {
			return $this->getSiteTree($markTrail, $activePageId, $lang);
		}


		// pokud není definován rodič a je omezená hloubka, vypomůžeme si algoritmem z getSiteTree
		if ($parentId === null) {

			// nebudeme po metodě chtít vyznačit trail, protože by se zbytečně rekurzí procházel celý strom webu (nám stačí jen do určité hloubky)
			$siteSubtree = $this->getSiteTree(false, null, $lang);

			if (isset($this->cache['SiteSubtree.null.'.$lang.'.'.$depth])) {
				$siteSubtree = $this->cache['SiteSubtree.null.'.$lang.'.'.$depth];
			} else {
				$siteSubtree = $this->limitTreeDepth($siteSubtree, $depth);

				$this->cache->save('SiteSubtree.null.'.$lang.'.'.$depth, $siteSubtree, array(
					Cache::TAGS => array('SiteTree.'.$lang)
				));
			}

			if ($markTrail !== false) {
				if ($activePageId === null) {
					$activePageId = $this->siteState['pageId'];
				}
				$ancestors = array_keys($this->getAncestors(false, $activePageId));
				$this->markTrail($siteSubtree, $ancestors, $activePageId);
			}

			return $siteSubtree;
		}

		$siteSubtree = array();

		$cacheDepth = $depth === null ? 'null' : $depth;

		if (isset($this->cache['SiteSubtree.'.$parentId.'.'.$cacheDepth])) {
			$siteSubtree = $this->cache['SiteSubtree.'.$parentId.'.'.$cacheDepth];
		} else {

			$parent = dibi::fetch('
				SELECT [lang], [family], [left], [right]
				FROM [page]
				WHERE [id] = %i', $parentId
			);

			$pages = dibi::fetchAll('
				SELECT [id] [pageId], [name], [homepage], /*[lang], */[url], [parent], [left], [right], [published], [shownInMenu]
				FROM [page]
				WHERE [left] BETWEEN %i', $parent['left'], ' AND %i', $parent['right'], '
				AND [family] = %i', $parent['family'], '
				ORDER BY [left]'
			);


			$fullSiteSubtree = array();

			if (!empty($pages)) {
				$fullSiteSubtree[] = $this->treeToArray($pages);
			}

			// snížení stromu o společného rodiče
			if ($depth !== null) {
				$fullSiteSubtree = $this->limitTreeDepth($fullSiteSubtree, $depth + 1);
			}
			if (isset($fullSiteSubtree[0]['children'])) {
				foreach ($fullSiteSubtree[0]['children'] as $tree) {
					$siteSubtree[] = $tree;
				}
			}

			$this->cache->save('SiteSubtree.'.$parentId.'.'.$cacheDepth, $siteSubtree, array(
				Cache::TAGS => array('SiteTree.'.$parent['lang'])
			));

		}

		if ($markTrail !== false) {
			if ($activePageId === null) {
				$activePageId = $this->siteState['pageId'];
			}
			$ancestors = array_keys($this->getAncestors(false, $activePageId));
			$this->markTrail($siteSubtree, $ancestors, $activePageId);
		}
		return $siteSubtree;


	}



	public function getFlatSiteTree($lang = null)
	{

		if ($lang === null) {
			$lang = $this->siteState['lang'];
		}

		$siteTree = $this->getSiteTree(false, null, $lang);

		$flatSiteTree = array();
		$this->createFlatTree($siteTree, $flatSiteTree);

		return $flatSiteTree;
	}


	private function createFlatTree($siteTree, &$flatSiteTree, $level = 0)
	{
		$level++;
		foreach ($siteTree as $page) {
			$flatSiteTree[] = (array('pageId' => $page['pageId'], 'name' => $page['name'],'level' => $level, 'url' => $page['url']));
			if (isset($page['children'])) {
				$this->createFlatTree($page['children'], $flatSiteTree, $level);
			}
		}
	}


	private function markTrail(&$siteTree, $ancestors, $activePageId)
	{
		foreach ($siteTree as $index => &$page) {
			if (in_array($page['pageId'], $ancestors)) {
				$page['trail'] = true;
			}
			if ($page['pageId'] == $activePageId) {
				$page['active'] = true;
			}
			if (isset($page['children'])) {
				$this->markTrail($page['children'], $ancestors, $activePageId);
			}
		}
	}



	private function limitTreeDepth($tree, $depth)
	{
		$subTree = array();

		foreach ($tree as $index => $page) {

			if ($depth > 0 or $depth === null) {

				$children = isset($page['children']) ? $page['children'] : null;
				unset($page['children']);

				$subTree[$index] = $page;
				if ($children !== null) {

					$innerDepth = $depth !== null ? $depth - 1 : null;
					$grandchildren = $this->limitTreeDepth($children, $innerDepth);
					if (!empty($grandchildren)) {
						$subTree[$index]['children'] = $grandchildren;
					}
				}
			}
		}
		return $subTree;
	}


	/**
	 * Volá se rekurzivně při vytváření vícerozměrného pole ze stromu v databázi
	 */
	private function treeToArray($pages)
	{
		$tree = $pages[0];

		array_shift($pages);

		if ($tree['left'] + 1 != $tree['right']) {

			foreach ($pages as $key => $page) {

				if ($page['parent'] != $tree['pageId']) {
					continue;
				}
				$tree['children'][] = $this->treeToArray($pages);

				foreach ($pages as $child_key => $child) {
					if ($child['right'] < $page['right'])
						unset($pages[$child_key]);
				}
				array_shift($pages);

			}
		}

		return (array) $tree;
	}



	public function normalizeOrders($parent)
	{
		$query = dibi::select('[id], [lang]')->from('page');
		if ($parent === null) {
			$query->where('parent')->isNull();
		} else {
			$query->where('[parent] = %i', $parent);
		}
		$ord = 1;
		$lang = null;
		foreach ($query->orderBy('ord')->fetchAll() as $page) {
			$lang = $page['lang'];
			dibi::query('UPDATE [page] SET [ord] = %i', $ord, 'WHERE [id] = %i', $page['id']);
			$ord++;
		}

		/*$this->cache->clean(array(
			Cache::TAGS => array('SiteTree.'.$lang)
		));*/
	}




	/************ metody pro generování traverzovatelných stromů ************/

	/**
	 * Re-generuje traverzovatelné stromy v tabulce page
	 *
	 */
	public function rebuildTraversalTrees($lang)
	{
		foreach (dibi::query('
			SELECT [id] [pageId]
			FROM [page]
			WHERE [parent] IS NULL AND [lang] = %s', $lang, '
			ORDER BY [ord]
		') as $page) {
			// koncepce webu je taková, že na nejvyšší úrovni je určitý počet stránek, z nichž každá definuje svůj vlastní strom
			$this->rebuildTraversalTree($page['pageId'], $page['pageId'], 1, $lang);
		}

		$this->cache->clean(array(
			Cache::TAGS => array('SiteTree.'.$lang)
		));
	}


	/**
	 * Re-generuje jeden konkrétní traverzovatelný strom
	 *
	 */
	private function rebuildTraversalTree($parent, $family, $left, $lang)
	{
		$right = $left + 1;
		foreach (dibi::query('
			SELECT [id] [pageId]
			FROM [page]
			WHERE [parent] = %i', $parent, ' AND [lang] = %s', $lang, '
			ORDER BY [ord]
		') as $page) {
			$right = $this->rebuildTraversalTree($page['pageId'], $family, $right, $lang);
		}
		dibi::query('
			UPDATE [page] SET [left] = %i', $left, ', [right] = %i', $right, ', [family] = %i', $family, '
			WHERE [id] = %i', $parent, ' AND [lang] = %s', $lang
		);
		return $right + 1;
	}



}